@@ -28,7 +28,6 @@
 #include <unistd.h>
 #include <sys/prctl.h>
 #include <sys/time.h>
-#include <sys/resource.h>
 
 #ifdef THREAD_DUMP
 #ifdef _WITH_SNMP_
@@ -152,8 +151,6 @@ checker_ipvs_syncd_needed(void)
 static int
 checker_terminate_phase2(void)
 {
-	struct rusage usage;
-
 	/* Remove the notify fifo */
 	notify_fifo_close(&global_data->notify_fifo, &global_data->lvs_notify_fifo);
 
@@ -191,12 +188,7 @@ checker_terminate_phase2(void)
 	 * Reached when terminate signal catched.
 	 * finally return to parent process.
 	 */
-	if (__test_bit(LOG_DETAIL_BIT, &debug)) {
-		getrusage(RUSAGE_SELF, &usage);
-		log_message(LOG_INFO, "Stopped - used %ld.%6.6ld user time, %ld.%6.6ld system time", usage.ru_utime.tv_sec, usage.ru_utime.tv_usec, usage.ru_stime.tv_sec, usage.ru_stime.tv_usec);
-	}
-	else
-		log_message(LOG_INFO, "Stopped");
+	log_stopping();
 
 #ifdef ENABLE_LOG_TO_FILE
 	if (log_file_name)
@@ -362,10 +354,6 @@ start_check(list_head_t *old_checkers_queue, data_t *prev_global_data)
 		return;
 	}
 
-#ifdef _MEM_CHECK_
-	log_message(LOG_INFO, "Configuration is using : %zu Bytes", mem_allocated);
-#endif
-
 	/* If we are just testing the configuration, then we terminate now */
 	if (__test_bit(CONFIG_TEST_BIT, &debug))
 		return;
@@ -489,6 +477,12 @@ reload_check_thread(__attribute__((unused)) thread_ref_t thread)
 	/* Use standard scheduling while reloading */
 	reset_process_priorities();
 
+#ifndef _ONE_PROCESS_DEBUG_
+	save_config(false, "check", dump_data_check);
+#endif
+
+	reinitialise_global_vars();
+
 	/* set the reloading flag */
 	SET_RELOAD;
 
@@ -531,7 +525,16 @@ reload_check_thread(__attribute__((unused)) thread_ref_t thread)
 	free_check_data(old_check_data);
 	free_global_data(old_global_data);
 	free_checker_list(&old_checkers_queue);
+
+#ifndef _ONE_PROCESS_DEBUG_
+	save_config(true, "check", dump_data_check);
+#endif
+
 	UNSET_RELOAD;
+
+#ifdef _MEM_CHECK_
+	log_message(LOG_INFO, "Configuration is using : %zu Bytes", mem_allocated);
+#endif
 }
 
 static void
@@ -775,6 +778,10 @@ start_check_child(void)
 	register_check_thread_addresses();
 #endif
 
+#ifdef _MEM_CHECK_
+	log_message(LOG_INFO, "Configuration is using : %zu Bytes", mem_allocated);
+#endif
+
 	/* Launch the scheduling I/O multiplexer */
 	launch_thread_scheduler(master);
 
