@@ -104,6 +104,7 @@ set_vrrp_defaults(data_t * data)
 	data->vrrp_garp_delay = VRRP_GARP_DELAY;
 	data->vrrp_garp_lower_prio_delay = PARAMETER_UNSET;
 	data->vrrp_garp_lower_prio_rep = PARAMETER_UNSET;
+	data->vrrp_down_timer_adverts = VRRP_DOWN_TIMER_ADVERTS;
 #ifdef _HAVE_VRRP_VMAC_
 	data->vrrp_vmac_garp_intvl = 0;
 #endif
@@ -416,6 +417,46 @@ free_global_data(data_t * data)
 	FREE(data);
 }
 
+FILE * __attribute__((malloc))
+open_dump_file(const char *file_name)
+{
+	FILE *fp;
+	char *file_name_tmp = NULL;
+	const char *dot;
+	int len;
+
+	if (global_data->data_use_instance &&
+	    (global_data->instance_name || global_data->network_namespace)) {
+		len = strlen(file_name) + 1;
+		if (global_data->instance_name)
+			len += strlen(global_data->instance_name) + 1;
+		if (global_data->network_namespace)
+			len += strlen(global_data->network_namespace) + 1;
+
+		file_name_tmp = MALLOC(len);
+
+		dot = strrchr(file_name, '.');
+		sprintf(file_name_tmp, "%.*s.%s%s%s%s",
+				(int)(dot - file_name), file_name,
+				global_data->network_namespace ? global_data->network_namespace : "",
+				global_data->instance_name && global_data->network_namespace ? "_" : "",
+				global_data->instance_name ? global_data->instance_name : "",
+				dot);
+		file_name = file_name_tmp;
+	}
+
+	fp = fopen_safe(file_name, "w");
+
+	if (!fp)
+		log_message(LOG_INFO, "Can't open dump file %s (%d: %s)",
+			file_name, errno, strerror(errno));
+
+	if (file_name_tmp)
+		FREE(file_name_tmp);
+
+	return fp;
+}
+
 void
 dump_global_data(FILE *fp, data_t * data)
 {
@@ -434,6 +475,11 @@ dump_global_data(FILE *fp, data_t * data)
 
 	conf_write(fp, "------< Global definitions >------");
 
+#ifndef _ONE_PROCESS_DEBUG_
+	if (config_save_dir)
+		conf_write(fp, " Config save dir = %s", config_save_dir);
+#endif
+
 	conf_write(fp, " Network namespace = %s", data->network_namespace ? data->network_namespace : "(default)");
 	conf_write(fp, " Network namespace ipvs = %s", data->network_namespace_ipvs ? data->network_namespace_ipvs[0] ? data->network_namespace_ipvs : "(default)" : "(main namespace)");
 	if (data->instance_name)
@@ -499,6 +545,8 @@ dump_global_data(FILE *fp, data_t * data)
 #endif
 	if (data->config_directory)
 		conf_write(fp, " config save directory = %s", data->config_directory);
+	if (data->data_use_instance)
+		conf_write(fp, " Use instance name in data dumps");
 	if (data->startup_script)
 		conf_write(fp, " Startup script = %s, uid:gid %u:%u, timeout %u",
 			    cmd_str(data->startup_script),
@@ -603,6 +651,7 @@ dump_global_data(FILE *fp, data_t * data)
 	conf_write(fp, " Gratuitous ARP refresh repeat = %u", data->vrrp_garp_refresh_rep);
 	conf_write(fp, " Gratuitous ARP lower priority delay = %u", data->vrrp_garp_lower_prio_delay == PARAMETER_UNSET ? PARAMETER_UNSET : data->vrrp_garp_lower_prio_delay / TIMER_HZ);
 	conf_write(fp, " Gratuitous ARP lower priority repeat = %u", data->vrrp_garp_lower_prio_rep);
+	conf_write(fp, " Num adverts before down = %u", data->vrrp_down_timer_adverts);
 #ifdef _HAVE_VRRP_VMAC_
 	if (data->vrrp_vmac_garp_intvl != PARAMETER_UNSET)
 		conf_write(fp, " Gratuitous ARP for each secondary %s = %us", data->vrrp_vmac_garp_all_if ? "i/f" : "VMAC", data->vrrp_vmac_garp_intvl);
