@@ -133,6 +133,10 @@ typedef struct _interface {
 	list_head_t		sin6_addr_l;		/* List of extra IPv6 interface addresses - sin_addr_t */
 #endif
 	unsigned		ifi_flags;		/* Kernel flags */
+	bool			seen_up;		/* True once we have first seen the interface up */
+	thread_ref_t		flags_change_thread;
+	unsigned		up_debounce_timer;
+	unsigned		down_debounce_timer;
 	uint32_t		mtu;			/* MTU for this interface_t */
 	unsigned short		hw_type;		/* Type of hardware address */
 	u_char			hw_addr[MAX_ADDR_LEN];	/* MAC address */
@@ -145,6 +149,9 @@ typedef struct _interface {
 #ifdef _HAVE_VRRP_VMAC_
 	if_type_t		if_type;		/* interface type */
 	int			vmac_type;		/* Type of macvlan or ipvlan */
+#ifdef HAVE_DECL_IFLA_IPVLAN_FLAGS
+	int			ipvlan_flags;		/* bridge/private/vepa */
+#endif
 	ifindex_t		base_ifindex;		/* Only used at startup if we find vmac i/f before base i/f */
 #ifdef HAVE_IFLA_LINK_NETNSID
 	int			base_netns_id;		/* Network namespace of the parent interface */
