@@ -29,7 +29,6 @@
 #include <unistd.h>
 #include <sys/prctl.h>
 #include <sys/time.h>
-#include <sys/resource.h>
 #include <fcntl.h>
 
 #ifdef THREAD_DUMP
@@ -261,8 +260,6 @@ reset_disable_local_igmp(void)
 static int
 vrrp_terminate_phase2(int exit_status)
 {
-	struct rusage usage;
-
 #ifdef _NETLINK_TIMERS_
 	if (do_netlink_timers)
 		report_and_clear_netlink_timers("Starting shutdown instances");
@@ -334,12 +331,7 @@ vrrp_terminate_phase2(int exit_status)
 	 * Reached when terminate signal catched.
 	 * finally return to parent process.
 	 */
-	if (__test_bit(LOG_DETAIL_BIT, &debug)) {
-		getrusage(RUSAGE_SELF, &usage);
-		log_message(LOG_INFO, "Stopped - used %ld.%6.6ld user time, %ld.%6.6ld system time", usage.ru_utime.tv_sec, usage.ru_utime.tv_usec, usage.ru_stime.tv_sec, usage.ru_stime.tv_usec);
-	}
-	else
-		log_message(LOG_INFO, "Stopped");
+	log_stopping();
 
 #ifdef ENABLE_LOG_TO_FILE
 	if (log_file_name)
@@ -664,11 +656,6 @@ start_vrrp(data_t *prev_global_data)
 		dbus_stop();
 #endif
 
-	/* Post initializations */
-#ifdef _MEM_CHECK_
-	log_message(LOG_INFO, "Configuration is using : %zu Bytes", mem_allocated);
-#endif
-
 	/* Set static entries */
 	netlink_iplist(&vrrp_data->static_addresses, IPADDRESS_ADD, false);
 	netlink_rtlist(&vrrp_data->static_routes, IPROUTE_ADD, false);
@@ -798,6 +785,12 @@ reload_vrrp_thread(__attribute__((unused)) thread_ref_t thread)
 	/* Use standard scheduling while reloading */
 	reset_process_priorities();
 
+#ifndef _ONE_PROCESS_DEBUG_
+	save_config(false, "vrrp", dump_data_vrrp);
+#endif
+
+	reinitialise_global_vars();
+
 	/* set the reloading flag */
 	SET_RELOAD;
 
@@ -872,7 +865,16 @@ reload_vrrp_thread(__attribute__((unused)) thread_ref_t thread)
 
 	free_old_interface_queue();
 
+#ifndef _ONE_PROCESS_DEBUG_
+	save_config(true, "vrrp", dump_data_vrrp);
+#endif
+
 	UNSET_RELOAD;
+
+	/* Post initializations */
+#ifdef _MEM_CHECK_
+	log_message(LOG_INFO, "Configuration is using : %zu Bytes", mem_allocated);
+#endif
 }
 
 static void
@@ -1108,6 +1110,14 @@ start_vrrp_child(void)
 	register_vrrp_thread_addresses();
 #endif
 
+	/* Post initializations */
+#ifdef _MEM_CHECK_
+	/* Note: there may be a proc_events_ack_timer thread which will not
+	 * exist when the same configuration is reloaded. This is a thread_t,
+	 * which currently adds 120 bytes to the allocated memory. */
+	log_message(LOG_INFO, "Configuration is using : %zu Bytes", mem_allocated);
+#endif
+
 #ifdef _WITH_PERF_
 	if (perf_run == PERF_RUN)
 		run_perf("vrrp", global_data->network_namespace, global_data->instance_name);
