@@ -1088,7 +1088,7 @@ nft_update_ipv6_address(struct mnl_nlmsg_batch *batch, ip_address_t *addr, bool
 static void
 nft_update_addresses(const vrrp_t *vrrp, int cmd)
 {
-	struct mnl_nlmsg_batch *batch;
+	struct mnl_nlmsg_batch *batch = NULL;
 	struct nlmsghdr *nlh;
 	ip_address_t *ip_addr;
 	struct nftnl_set *ipv4_set = NULL;
@@ -1101,13 +1101,14 @@ nft_update_addresses(const vrrp_t *vrrp, int cmd)
 	const list_head_t *vip_list;
 	int proto;
 
-	batch = nft_start_batch();
-
 	for (vip_list = &vrrp->vip; vip_list; vip_list = vip_list == &vrrp->vip ? &vrrp->evip : NULL) {
 		list_for_each_entry(ip_addr, vip_list, e_list) {
 			if (set_rule == ip_addr->nftable_rule_set)
 				continue;
 
+			if (!batch)
+				batch = nft_start_batch();
+
 			if (ip_addr->ifa.ifa_family == AF_INET)
 				nft_update_ipv4_address(batch, ip_addr, &ipv4_set);
 			else
@@ -1118,6 +1119,9 @@ nft_update_addresses(const vrrp_t *vrrp, int cmd)
 		}
 	}
 
+	if (!batch)
+		return;
+
 	for (ip_set = &ipv4_set, proto = NFPROTO_IPV4; ip_set;
 	     ip_set = 
 		ip_set == &ipv4_set ? &ipv6_set :
@@ -1443,6 +1447,10 @@ nft_cleanup(void)
 void
 nft_end(void)
 {
+
+	if (!nl)
+		return;
+
 	nft_cleanup();
 
 	mnl_socket_close(nl);
