@@ -850,16 +850,16 @@ static void
 vrrp_adv_handler(const vector_t *strvec)
 {
 	vrrp_t *vrrp = list_last_entry(&vrrp_data->vrrp, vrrp_t, e_list);
-	double adver_int;
+	unsigned adver_int;
 	bool res;
 
-	res = read_double_strvec(strvec, 1, &adver_int, 0.01F, 255.0F, true);
+	res = read_decimal_unsigned_strvec(strvec, 1, &adver_int, TIMER_HZ / 100, 255 * TIMER_HZ, TIMER_HZ_DIGITS, true);
 
 	/* Simple check - just positive */
 	if (!res || adver_int <= 0)
 		report_config_error(CONFIG_GENERAL_ERROR, "(%s) Advert interval (%s) not valid! Must be > 0 - ignoring", vrrp->iname, strvec_slot(strvec, 1));
 	else
-		vrrp->adver_int = (unsigned)(adver_int * TIMER_HZ);
+		vrrp->adver_int = adver_int;
 }
 static void
 vrrp_debug_handler(const vector_t *strvec)
@@ -922,14 +922,14 @@ static void
 vrrp_preempt_delay_handler(const vector_t *strvec)
 {
 	vrrp_t *vrrp = list_last_entry(&vrrp_data->vrrp, vrrp_t, e_list);
-	double preempt_delay;
+	unsigned preempt_delay;
 
-	if (!read_double_strvec(strvec, 1, &preempt_delay, 0, TIMER_MAX_SEC, true)) {
+	if (!read_decimal_unsigned_strvec(strvec, 1, &preempt_delay, 0, TIMER_MAX_SEC * TIMER_HZ, TIMER_HZ_DIGITS, true)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "(%s) Preempt_delay not valid! must be between 0-%u", vrrp->iname, TIMER_MAX_SEC);
 		vrrp->preempt_delay = 0;
 	}
 	else
-		vrrp->preempt_delay = (unsigned long)(preempt_delay * TIMER_HZ);
+		vrrp->preempt_delay = preempt_delay;
 }
 static void
 vrrp_notify_backup_handler(const vector_t *strvec)
@@ -1156,25 +1156,44 @@ vrrp_garp_lower_prio_rep_handler(const vector_t *strvec)
 
 	vrrp->garp_lower_prio_rep = garp_lower_prio_rep;
 }
+static void
+vrrp_down_timer_adverts_handler(const vector_t *strvec)
+{
+	vrrp_t *vrrp = list_last_entry(&vrrp_data->vrrp, vrrp_t, e_list);
+	unsigned down_timer_adverts;
+
+	if (!read_unsigned_strvec(strvec, 1, &down_timer_adverts, 1, 100, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "(%s): Invalid down_timer_adverts [1:100] '%s'", vrrp->iname, strvec_slot(strvec, 1));
+		return;
+	}
+
+	vrrp->down_timer_adverts = down_timer_adverts;
+}
 #ifdef _HAVE_VRRP_VMAC_
 static void
-vrrp_vmac_garp_intvl_handler(const vector_t *strvec)
+vrrp_garp_extra_if_handler(const vector_t *strvec)
 {
 	vrrp_t *vrrp = list_last_entry(&vrrp_data->vrrp, vrrp_t, e_list);
 	unsigned delay = UINT_MAX;
 	unsigned index;
+	const char *cmd_name = strvec_slot(strvec, 0);
+
+	if (!strcmp(cmd_name, "vmac_garp_intvl")) {
+		/* Deprecated after v2.2.2 */
+		report_config_error(CONFIG_DEPRECATED, "Keyword \"vmac_garp_intvl\" is deprecated - please use \"garp_extra_if\"");
+	}
 
 	for (index = 1; index < vector_size(strvec); index++) {
 		if (!strcmp(strvec_slot(strvec, index), "all"))
 			vrrp->vmac_garp_all_if = true;
 		else if (!read_unsigned_strvec(strvec, index, &delay, 0, 86400, true)) {
-			report_config_error(CONFIG_GENERAL_ERROR, "(%s): vmac_garp_intvl '%s' invalid - ignoring", vrrp->iname, strvec_slot(strvec, index));
+			report_config_error(CONFIG_GENERAL_ERROR, "(%s): %s '%s' invalid - ignoring", vrrp->iname, cmd_name, strvec_slot(strvec, index));
 			return;
 		}
 	}
 
 	if (delay == UINT_MAX) {
-		report_config_error(CONFIG_GENERAL_ERROR, "(%s): vmac_garp_intvl specified without time - ignoring", vrrp->iname);
+		report_config_error(CONFIG_GENERAL_ERROR, "(%s): %s specified without time - ignoring", vrrp->iname, cmd_name);
 		return;
 	}
 
@@ -1592,9 +1611,9 @@ static void
 vrrp_tprocess_delay_general(const vector_t *strvec, enum process_delay delay_type)
 {
 	vrrp_tracked_process_t *tprocess = list_last_entry(&vrrp_data->vrrp_track_processes, vrrp_tracked_process_t, e_list);
-	double delay;
+	unsigned delay;
 
-	if (!read_double_strvec(strvec, 1, &delay, 0.000001F, 3600.F, true)) {
+	if (!read_decimal_unsigned_strvec(strvec, 1, &delay, 1, 3600U * TIMER_HZ, TIMER_HZ_DIGITS, true)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "%sdelay (%s) for vrrp_track_process %s must be between "
 							  "[0.000001..3600] inclusive. Ignoring..."
 							, delay_type == PROCESS_TERMINATE_DELAY ? "terminate_" :
@@ -1604,9 +1623,9 @@ vrrp_tprocess_delay_general(const vector_t *strvec, enum process_delay delay_typ
 	}
 
 	if (delay_type != PROCESS_FORK_DELAY)
-		tprocess->terminate_delay = (unsigned)(delay * TIMER_HZ);
+		tprocess->terminate_delay = delay;
 	if (delay_type != PROCESS_TERMINATE_DELAY)
-		tprocess->fork_delay = (unsigned)(delay * TIMER_HZ);
+		tprocess->fork_delay = delay;
 }
 static void
 vrrp_tprocess_terminate_delay_handler(const vector_t *strvec)
@@ -1714,15 +1733,15 @@ static void
 garp_group_garp_interval_handler(const vector_t *strvec)
 {
 	garp_delay_t *delay = list_last_entry(&garp_delay, garp_delay_t, e_list);
-	double val;
+	unsigned val;
 
-	if (!read_double_strvec(strvec, 1, &val, 0, (int)(INT_MAX / 1000000), true)) {
+	if (!read_decimal_unsigned_strvec(strvec, 1, &val, 0, INT_MAX, TIMER_HZ_DIGITS, true)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "garp_group garp_interval '%s' invalid", strvec_slot(strvec, 1));
 		return;
 	}
 
-	delay->garp_interval.tv_sec = (time_t)val;
-	delay->garp_interval.tv_usec = (suseconds_t)((val - delay->garp_interval.tv_sec) * 1000000);
+	delay->garp_interval.tv_sec = (time_t)val / TIMER_HZ;
+	delay->garp_interval.tv_usec = (suseconds_t)(val % TIMER_HZ);
 	delay->have_garp_interval = true;
 
 	if (delay->garp_interval.tv_sec >= 1)
@@ -1732,15 +1751,15 @@ static void
 garp_group_gna_interval_handler(const vector_t *strvec)
 {
 	garp_delay_t *delay = list_last_entry(&garp_delay, garp_delay_t, e_list);
-	double val;
+	unsigned val;
 
-	if (!read_double_strvec(strvec, 1, &val, 0, (int)(INT_MAX / 1000000), true)) {
+	if (!read_decimal_unsigned_strvec(strvec, 1, &val, 0, INT_MAX, TIMER_HZ_DIGITS, true)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "garp_group gna_interval '%s' invalid", strvec_slot(strvec, 1));
 		return;
 	}
 
-	delay->gna_interval.tv_sec = (time_t)val;
-	delay->gna_interval.tv_usec = (suseconds_t)((val - delay->gna_interval.tv_sec) * 1000000);
+	delay->gna_interval.tv_sec = (time_t)val / TIMER_HZ;
+	delay->gna_interval.tv_usec = (suseconds_t)(val % TIMER_HZ);
 	delay->have_gna_interval = true;
 
 	if (delay->gna_interval.tv_sec >= 1)
@@ -1838,6 +1857,59 @@ garp_group_end_handler(void)
 	}
 }
 
+static void
+alloc_if_up_down_delay(const vector_t *strvec)
+{
+	unsigned down_delay = 0;
+	unsigned up_delay = 0;
+	int res;
+	unsigned long delay;
+	interface_t *ifp;
+
+	if (!(ifp = if_get_by_ifname(strvec_slot(strvec, 0), global_data->dynamic_interfaces))) {
+		report_config_error(CONFIG_FATAL, "unknown interface %s specified for up/down delay", strvec_slot(strvec, 0));
+		return;
+	}
+
+	if (vector_size(strvec) < 2) {
+		log_message(LOG_INFO, "No timeouts specified for %s up/down delays", ifp->ifname);
+		return;
+	}
+	if (vector_size(strvec) > 3)
+		log_message(LOG_INFO, "Too many parameters for %s up/down delays", ifp->ifname);
+
+	res = read_timer(strvec, 1, &delay, 0, 255 * TIMER_HZ, true);
+	if (!res) {
+		log_message(LOG_INFO, "Invalid down delay %s for %s", strvec_slot(strvec, 1), ifp->ifname);
+		return;
+	}
+	down_delay = (unsigned)delay;
+
+	if (vector_size(strvec) == 2)
+		up_delay = 0;
+	else {
+		res = read_timer(strvec, 2, &delay, 0, 255 * TIMER_HZ, true);
+		if (!res) {
+			log_message(LOG_INFO, "Invalid up delay %s for %s", strvec_slot(strvec, 2), ifp->ifname);
+			return;
+		}
+		up_delay = (unsigned)delay;
+	}
+
+	ifp->down_debounce_timer = down_delay;
+	ifp->up_debounce_timer = up_delay;
+}
+
+/* interface state change delays handler */
+static void
+interface_up_down_delays_handler(const vector_t *strvec)
+{
+	if (!strvec)
+		return;
+
+	alloc_value_block(alloc_if_up_down_delay, strvec);
+}
+
 void
 init_vrrp_keywords(bool active)
 {
@@ -1958,8 +2030,10 @@ init_vrrp_keywords(bool active)
 	install_keyword("garp_master_refresh_repeat", &vrrp_garp_refresh_rep_handler);
 	install_keyword("garp_lower_prio_delay", &vrrp_garp_lower_prio_delay_handler);
 	install_keyword("garp_lower_prio_repeat", &vrrp_garp_lower_prio_rep_handler);
+	install_keyword("down_timer_adverts", &vrrp_down_timer_adverts_handler);
 #ifdef _HAVE_VRRP_VMAC_
-	install_keyword("vmac_garp_intvl", &vrrp_vmac_garp_intvl_handler);
+	install_keyword("garp_extra_if", &vrrp_garp_extra_if_handler);
+	install_keyword("vmac_garp_intvl", &vrrp_garp_extra_if_handler);	/* Deprecated after v2.2.2 - incorrect keyword in commit 3dcd13c */
 #endif
 	install_keyword("lower_prio_no_advert", &vrrp_lower_prio_no_advert_handler);
 	install_keyword("higher_prio_send_advert", &vrrp_higher_prio_send_advert_handler);
@@ -1997,6 +2071,9 @@ init_vrrp_keywords(bool active)
 	install_keyword("full_command", &vrrp_tprocess_full_handler);
 	install_sublevel_end_handler(&vrrp_tprocess_end_handler);
 #endif
+
+	/* Interface up down delays */
+	install_keyword_root("interface_up_down_delays", &interface_up_down_delays_handler, active);
 }
 
 const vector_t *
