@@ -578,7 +578,7 @@ vrrp_set_effective_priority(vrrp_t *vrrp)
 
 	vrrp->effective_priority = new_prio;
 	old_down_timer = vrrp->ms_down_timer;
-	vrrp->ms_down_timer = 3 * vrrp->master_adver_int + VRRP_TIMER_SKEW(vrrp);
+	vrrp->ms_down_timer = VRRP_MS_DOWN_TIMER(vrrp);
 
 	if (vrrp->state == VRRP_STATE_BACK) {
 		if (old_down_timer < vrrp->ms_down_timer)
@@ -837,6 +837,10 @@ initialise_vrrp_tracking_priorities(vrrp_t *vrrp)
 	if (vrrp->sync) {
 		list_for_each_entry(tsc, &vrrp->sync->track_script, e_list)
 			initialise_track_script_state(tsc, vrrp);
+#ifdef _WITH_BFD_
+		list_for_each_entry(tbfd, &vrrp->sync->track_bfd, e_list)
+			initialise_track_bfd_state(tbfd, vrrp);
+#endif
 	}
 
 	vrrp_set_effective_priority(vrrp);
