@@ -188,12 +188,6 @@ memcheck_ptr_cmp(const MEMCHECK *m1, const MEMCHECK *m2)
 	return (char *)m1->ptr - (char *)m2->ptr;
 }
 
-static inline int
-memcheck_seq_cmp(const MEMCHECK *m1, const MEMCHECK *m2)
-{
-	return m1->seq_num - m2->seq_num;
-}
-
 static const char *
 format_time(void)
 {
