@@ -58,6 +58,7 @@ typedef enum {
 	THREAD_CHILD_TERMINATED,
 	THREAD_TERMINATE_START,
 	THREAD_TERMINATE,
+	THREAD_READY_TIMER,
 	THREAD_READY_READ_FD,
 	THREAD_READY_WRITE_FD,
 	THREAD_READ_ERROR,
@@ -69,20 +70,37 @@ typedef enum {
 
 /* Thread Event flags */
 enum thread_flags {
-	THREAD_FL_READ_BIT,
-	THREAD_FL_WRITE_BIT,
-	THREAD_FL_EPOLL_BIT,
-	THREAD_FL_EPOLL_READ_BIT,
-	THREAD_FL_EPOLL_WRITE_BIT,
+	THREAD_FL_READ_BIT,		/* Want read set */
+	THREAD_FL_WRITE_BIT,		/* Want write set */
+	THREAD_FL_EPOLL_BIT,		/* fd is registered with epoll */
+	THREAD_FL_EPOLL_READ_BIT,	/* read is registered */
+	THREAD_FL_EPOLL_WRITE_BIT,	/* write is registered */
 };
 
 /* epoll def */
 #define THREAD_EPOLL_REALLOC_THRESH	64
 
+/* Thread flags for thread destruction */
+#define THREAD_DESTROY_CLOSE_FD	0x01
+#define THREAD_DESTROY_FREE_ARG	0x02
+
 typedef struct _thread thread_t;
 typedef const thread_t * thread_ref_t;
 typedef void (*thread_func_t)(thread_ref_t);
 
+typedef union {
+	int val;
+	unsigned uval;
+	struct {
+		int fd;		/* file descriptor in case of read/write. */
+		unsigned flags;
+	} f;
+	struct {
+		pid_t pid;	/* process id a child thread is wanting. */
+		int status;	/* return status of the process */
+	} c;
+} thread_arg2;
+
 /* Thread itself. */
 struct _thread {
 	unsigned long id;
@@ -91,17 +109,7 @@ struct _thread {
 	thread_func_t func;		/* event function */
 	void *arg;			/* event argument */
 	timeval_t sands;		/* rest of time sands value. */
-	union {
-		int val;		/* second argument of the event. */
-		struct {
-			int fd;		/* file descriptor in case of read/write. */
-			bool close_on_reload;
-		} f;
-		struct {
-			pid_t pid;	/* process id a child thread is wanting. */
-			int status;	/* return status of the process */
-		} c;
-	} u;
+	thread_arg2 u;			/* second argument of the event. */
 	struct _thread_event *event;	/* Thread Event back-pointer */
 
 	union {
@@ -242,14 +250,16 @@ extern void dump_thread_data(const thread_master_t *, FILE *);
 #endif
 extern void thread_cleanup_master(thread_master_t *);
 extern void thread_destroy_master(thread_master_t *);
-extern thread_ref_t thread_add_read_sands(thread_master_t *, thread_func_t, void *, int, const timeval_t *, bool);
-extern thread_ref_t thread_add_read(thread_master_t *, thread_func_t, void *, int, unsigned long, bool);
+extern thread_ref_t thread_add_read_sands(thread_master_t *, thread_func_t, void *, int, const timeval_t *, unsigned);
+extern thread_ref_t thread_add_read(thread_master_t *, thread_func_t, void *, int, unsigned long, unsigned);
 extern void thread_del_read(thread_ref_t);
 extern void thread_requeue_read(thread_master_t *, int, const timeval_t *);
-extern thread_ref_t thread_add_write(thread_master_t *, thread_func_t, void *, int, unsigned long, bool);
+extern thread_ref_t thread_add_write(thread_master_t *, thread_func_t, void *, int, unsigned long, unsigned);
 extern void thread_del_write(thread_ref_t);
 extern void thread_close_fd(thread_ref_t);
+extern thread_ref_t thread_add_timer_uval(thread_master_t *, thread_func_t, void *, unsigned, unsigned long);
 extern thread_ref_t thread_add_timer(thread_master_t *, thread_func_t, void *, unsigned long);
+extern void thread_update_arg2(thread_ref_t, const thread_arg2 *);
 extern void timer_thread_update_timeout(thread_ref_t, unsigned long);
 extern thread_ref_t thread_add_timer_shutdown(thread_master_t *, thread_func_t, void *, unsigned long);
 extern thread_ref_t thread_add_child(thread_master_t *, thread_func_t, void *, pid_t, unsigned long);
